import httpx

# API CGI base path
URL_CGI = "/recepcion_datos_4.cgi"

# API status operation code
API_STATUS_OP = 1002

LOCAL_TIMEOUT = httpx.Timeout(
    # The device can be slow to respond but fast to connect to we
    # need to set a long timeout for the read and a short timeout
    # for the connect
    timeout=10.0,
    read=60.0,
)



from pyecoforest.exceptions import EcoforestError
from pyecoforest.models.state import State
from pyecoforest.models.status import Status


def parse(response: str) -> Status:
    """Parse request data and return as status model."""
    data = _parse_text(response)
    return Status(
        on=data["on_off"] == "1",
        state=_parse_state(data["estado"]),
        power=int(data["consigna_potencia"]),
        temperature=float(data["temperatura"]),
    )


def _parse_text(response: str) -> dict[str, str]:
    """Parse request data and return as dictionary."""
    # discard last line ?
    reply = dict(e.split("=") for e in response.split("\n")[:-1])
    # Remove all white spaces from bad response from ecoforest ...
    reply = {x.translate({32: None}): y for x, y in reply.items()}
    return reply


def _parse_state(state: str) -> State:
    states = {
        "OFF": [0],
        "STARTING": [1, 2, 3, 4, 10],
        "PRE_HEATING": [5, 6],
        "ON": [7],
        "SHUTTING_DOWN": [8, 11, -3],
        "STAND_BY": [-20],
        "ALARM": [-4],
    }

    for k, v in states.items():
        if int(state) in v:
            return State[k]

    raise EcoforestError(f"The state {state} is not a valid state!")



"""Model for the Ecoforest stove status."""
from __future__ import annotations

from dataclasses import dataclass

from .state import State


@dataclass
class Status:
    """Model for the Ecoforest stove status."""

    on: bool
    state: State
    power: int
    temperature: float



import logging
from http import HTTPStatus
from typing import Any

import httpx

from pyecoforest.models.status import Status

from .const import API_STATUS_OP, LOCAL_TIMEOUT, URL_CGI
from .exceptions import EcoforestAuthenticationRequired
from .parser import parse
from .ssl import NO_VERIFY_SSL_CONTEXT

_LOGGER = logging.getLogger(__name__)


class EcoforestApi:
    """Class for communicating with an ecoforest device."""

    def __init__(
        self,
        host: str,
        auth: httpx.DigestAuth | None = None,
        client: httpx.AsyncClient | None = None,
        timeout: float | httpx.Timeout | None = None,
    ) -> None:
        self._host = host
        self._auth = auth
        # We use our own httpx client session so we can disable SSL verification,
        # the device use self-signed SSL certs.
        self._timeout = timeout or LOCAL_TIMEOUT
        self._client = client or httpx.AsyncClient(
            base_url=self._host, verify=NO_VERIFY_SSL_CONTEXT
        )  # nosec

    async def _request(self, data: dict[str, Any] | None = None) -> httpx.Response:
        """Make a request to the device."""
        if _LOGGER.isEnabledFor(logging.DEBUG):
            _LOGGER.debug("Sending POST to %s with data %s", URL_CGI, data)

        response = await self._client.post(
            URL_CGI,
            auth=self._auth,
            timeout=self._timeout,
            data=data,
        )

        status_code = response.status_code
        if status_code in (HTTPStatus.UNAUTHORIZED, HTTPStatus.FORBIDDEN):
            raise EcoforestAuthenticationRequired(
                f"Authentication failed for {URL_CGI} with status {status_code}, "
                "please check your username/password."
            )

        return parse(response.text)

    async def status(self) -> Status:
        """Retrieve ecoforest status."""
        result = await self._request(data={"idOperacion": API_STATUS_OP})
        return result



"""Model for the Ecoforest state."""

from enum import Enum


class State(Enum):
    """Model that represents the state of the device."""

    OFF = "off"
    STARTING = "starting"
    PRE_HEATING = "pre_heating"
    ON = "on"
    SHUTTING_DOWN = "shutting_down"
    STAND_BY = "stand_by"
    ALARM = "alarm"
